//fuses: reset disabled , 9.6MHz 

#include <avr/sleep.h>
#include <avr/wdt.h>

#define IN1 (PB5) //pin1 to INx power board
#define IN2 (PB3) //pin2 to INx power board
#define EN (PB4) // pin3 Led output to EN (is /SD in modified power board )
#define CurrSense (PB1) // pin6  detects Current with INT0 rising edge
#define CurrDir (PB2) // pin7 detect direction with PinChange PCINT2
#define CurrAttenuation (PB0) //pin5 , if input HIGH: detects Imin, if output LOW: detects Imax

//watchog delays, with clear IF, and IE interrupt enabled
#define _16msec 0b11000000 // 16msec 
#define _32msec 0b11000001
#define _64msec 0b11000010 
#define _125msec 0b11000011
#define _250msec 0b11000100 
#define _500msec 0b11000101
#define _1sec 0b11000110
#define _2secs   0b11000111
#define _4secs   0b11100000
#define _8secs   0b11100001 // 8 secs
uint8_t state=0;

volatile boolean wdt_flag=0, Min_detected_flag=0 ; 

void start_switching(void);
void stop_switching(void);

void setup() {
  // put your setup code here, to run once:
pinMode(EN, OUTPUT);
digitalWrite(EN, LOW); // (/SD in modified board) disables switching

pinMode(IN1, OUTPUT);
pinMode(IN2, OUTPUT);
digitalWrite(IN1, LOW);
digitalWrite(IN2, LOW);

pinMode(CurrAttenuation, INPUT); // Tristate for detecting low current
digitalWrite(CurrAttenuation, LOW); //detect strong current when pinMode OUTPUT 

pinMode(CurrDir, INPUT); // Tristate for detecting pinchange
digitalWrite(CurrDir, LOW); //lets see if lowers PB2 voltage 



cli();    // switch interrupts off while messing with their settings  

//*** Limiting Power Consumption***
  ADCSRA = 0;    // disable ADC
  ACSR= bit(ACD); // analog comparator disable
//  DIDR0= // Digital Input Disable Register not needed (only with ADC0-3)
// disable BOD in fuses? (+30uA @ 5V)  

// configure switching timer
  TCCR0A =  bit(WGM01);          // normal operation, CTC
    //Bits 7:6 – COM0A1:0: Compare Match Output A = 0,0 Mode = Normal port operation, OC0A disconnected
    //Bits 1:0 – WGM01:0: Waveform Generation Mode =1,0
  TCCR0B = bit(CS00);   //  No prescaling  
//  TCCR0B= 0b00000001; // Timer/Counter Control Register B
    //Bit 3 – WGM02: Waveform Generation Mode =0 -> WGM02:0: Waveform Generation Mode =0,1,0 Clear Timer on Compare Match (CTC) Mode
    //CS02:0: Clock Select = 0,0,1 : clk I/O /(No prescaling)
  OCR0A =  154;       // compare A register value equivalent to 256 (16MHz) here at 9.6MHz
  TIMSK0 = 0; // no timer0 interrupt enabled

// configure PinChange on PB2 to detect direction
  PCMSK= bit(PCINT2); //PCMSK – Pin Change Mask Register for PCINT2 on pin 7 (PB2)

// configure INT0 level change interrupt on PB1 for Direction
  MCUCR= 3 ; // The rising edge of INT0 generates an interrupt request
// no high current interrupt yet      
  GIMSK= 0 ; // Bit 6 – INT0: External Interrupt Request 0 DISabled

//configure Watchdog timer set to one second ***
  WDTCR |= 0b00011000;               // see docs, set WDCE, WDE
  WDTCR =  _1sec ;    //  WDIE enabled, delay set to 1 sec     

  sei(); //Enable global interrupts

}

void loop() {
  // put your main code here, to run repeatedly:
  if(TIMSK0) set_sleep_mode (SLEEP_MODE_IDLE);
  else set_sleep_mode (SLEEP_MODE_PWR_DOWN); // switching stopped
  sleep_enable();
  sleep_cpu ();            // here the device is put to sleep

//#####################SLEEPING############################33
  if(wdt_flag) {   // every Xsec
    
    switch (state) {
      case 0: // start 64msec switching with detect_low_current, changes to detect_high_current for 8secs in interrupt
        pinMode(CurrDir, OUTPUT); // Dir to ground, trying to discharge residual choke current
//detect_low_current();
//        pinMode(CurrAttenuation, INPUT); // transition from LOW to Tristate does'nt need intermediate state  
        Min_detected_flag=0;
        GIMSK= 0; // detection in switching interrupt
        WDTCR |= 0b00011000;               // see docs, set WDCE, WDE
        WDTCR =  _64msec ;    //  WDIE enabled, delay set
        wdt_reset();
        bitSet(GIFR,5); // reset PC interrupt flag (Direction)
        pinMode(CurrDir, INPUT);
        start_switching();
        state=1;
        break;
      case 1:
        if(!Min_detected_flag) { // Min not detected, stop switching and retry in 8secs
          stop_switching();
          WDTCR |= 0b00011000;               // see docs, set WDCE, WDE
          WDTCR =  _8secs ;    //  WDIE enabled, delay set
          wdt_reset();
          state=0;
        }
        else { // Min detected, go on switching for 4secs
          Min_detected_flag=0;
          WDTCR |= 0b00011000;               // see docs, set WDCE, WDE
          WDTCR =  _4secs ;    //  WDIE enabled, delay set
          wdt_reset();
          state=2;
        }  
        
        break;
      case 2: // 4seconds successful switching, stop for 0.125/0.250sec
        stop_switching();
        if(bitRead(GIFR,5)){ //Direction
          WDTCR |= 0b00011000;               // see docs, set WDCE, WDE
          WDTCR =  _125msec ;    //  WDIE enabled, delay set
          wdt_reset();
          state=3;
        }
        else {
          pinMode(CurrAttenuation, INPUT); // transition from LOW to Tristate does'nt need intermediate state  
          WDTCR |= 0b00011000;               // see docs, set WDCE, WDE
          WDTCR =  _250msec ;
          wdt_reset();
          state=0;
        }
        break;
      case 3: // start 64msec switching for blink
        WDTCR |= 0b00011000;               // see docs, set WDCE, WDE
        WDTCR =  _64msec ;    //  WDIE enabled, delay set
        wdt_reset();
        start_switching();
        state=4;
        break;
      case 4: // stop switching for 125msecs before new cycle
        stop_switching();
        pinMode(CurrAttenuation, INPUT); // transition from LOW to Tristate does'nt need intermediate state  
        WDTCR |= 0b00011000;               // see docs, set WDCE, WDE
        WDTCR =  _125msec ;    //  WDIE enabled, delay set
        wdt_reset();
        state=0;
        break;
    }    
    wdt_flag=0;
  }     

}


void start_switching(void) {
  bitSet(GIFR,6); // reset INT0 interrupt flag  for current detection
  //switching params
  // inverse outputs IN1,IN2
  digitalWrite(EN, LOW); // (/SD in modified board) disables switching  
  digitalWrite(IN1, HIGH);
  digitalWrite(IN2, LOW);
  // start timer0
  TIMSK0 = bit(OCIE0A); // interrupt on Compare A Match
  digitalWrite(EN, HIGH); // (/SD in modified board) enable switching 
}

void stop_switching(void){ // to minimum consumption DEEP SLEEP
  digitalWrite(EN, LOW); // (/SD in modified board) disables switching  
  // stop timer
  TIMSK0 = 0; // timer interrupt disabled
  digitalWrite(IN1, LOW); //to be sure,  do it after disabling switching 
  digitalWrite(IN2, LOW);
}  
